﻿using MSI.Afterburner;
using System;
using GpuFanControl.MsiAfterburnerWrap;

namespace GpuFanControl
{
    internal class Program
    {
        private static int Main(string[] args)
        {
            TryingSandBox.Go();

            return -9999;

            if (args.Length == 0)
            {
                PrintUsage();
                return -1;
            }


            if (args[0].ToLower().StartsWith("-q"))
            {
                int? index = null;
                if (int.TryParse(args[0].Substring(2), out int parsedIndex))
                {
                    index = parsedIndex;
                }

                var cm = ConnectToAfterburner(out var errCode);
                if (cm == null) return errCode;

                void PrintSpeedTableHeader() => Console.WriteLine("index\tmode\tmin\tval\tmax\t%");

                void PrintGpuFanSpeed(ControlMemoryGpuEntry g)
                {
                    if (!g.Flags.HasFlag(MACM_SHARED_MEMORY_GPU_ENTRY_FLAG.FAN_SPEED))
                    {
                        Console.WriteLine($"{g.Index}\t** No support fan speed **");
                    }
                    else
                    {
                        var gfr = g.FanSpeedMax - g.FanSpeedMin;
                        Console.WriteLine($"{g.Index}\t{g.FanFlagsCur}\t{g.FanSpeedMin}\t{g.FanSpeedCur}\t{g.FanSpeedMax}\t{((float)g.FanSpeedCur - g.FanSpeedMin) / gfr * 100:N2}%");
                    }
                }

                if (index.HasValue)
                {
                    if (index < 0 || index > cm.GpuEntries.Length - 1)
                    {
                        Console.WriteLine("Argument Error: index out of GPU entries");
                        return -1;
                    }
                    //show specified
                    PrintSpeedTableHeader();
                    var g = cm.GpuEntries[index.Value];
                    PrintGpuFanSpeed(g);
                }
                else
                {
                    //show all
                    PrintSpeedTableHeader();
                    foreach (var g in cm.GpuEntries)
                    {
                        PrintGpuFanSpeed(g);
                    }
                }
            }

            if (args[0].ToLower().StartsWith("-s"))
            {
                float? value = null;
                var sParam = args[0].Substring(2);
                if (sParam.ToLower() == "auto")
                {
                }
                else if (float.TryParse(sParam, out var parsedValue))
                {
                    value = parsedValue;
                }
                else
                {
                    Console.WriteLine("Argument Error: -s ");
                    return -1;
                }

                int? index = null;
                if (args.Length > 1 && args[1].ToLower().StartsWith("-i") && int.TryParse(args[1].Substring(2), out int parsedIndex))
                {
                    index = parsedIndex;
                }

                var cm = ConnectToAfterburner(out var errCode);
                if (cm == null) return errCode;

                void SetGpuFanSpeed(ControlMemoryGpuEntry g)
                {
                    Console.WriteLine($"Try change parameter for GPU {g.Index}");
                    try
                    {
                        if (value.HasValue)
                        {
                            g.FanFlagsCur = MACM_SHARED_MEMORY_GPU_ENTRY_FAN_FLAG.None;

                            var gfr = g.FanSpeedMax - g.FanSpeedMin;
                            var targetValue = (uint)Math.Round(g.FanSpeedMin + gfr * value.Value / 100);

                            if (targetValue < g.FanSpeedMin) g.FanSpeedCur = g.FanSpeedMin;
                            else if (targetValue > g.FanSpeedMax) g.FanSpeedCur = g.FanSpeedMax;
                            else g.FanSpeedCur = targetValue;

                            Console.WriteLine($"G{g.Index} min:{g.FanSpeedMin} max:{g.FanSpeedMax} set:{g.FanSpeedCur}");
                        }
                        else
                        {
                            g.FanFlagsCur = MACM_SHARED_MEMORY_GPU_ENTRY_FAN_FLAG.AUTO;
                        }

                        Console.WriteLine("Success.");
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine($"Failure to change parameter for GPU {g.Index}, {e.Message}");
                        return;
                    }

                    Console.WriteLine($"Commiting Change For GPU {g.Index}");
                    try
                    {
                        cm.CommitChanges(g.Index);
                        Console.WriteLine($"Success.");
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine($"Failure Commit Change For GPU {g.Index}, {ex.Message}");
                    }

                    Console.WriteLine($"Reloading Control Memory...");
                    try
                    {
                        cm.ReloadAll();
                        Console.WriteLine($"Success.");
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine($"Failure Reloading Control Memory, {e.Message}");
                    }
                }

                if (index.HasValue)
                {
                    if (index < 0 || index > cm.GpuEntries.Length - 1)
                    {
                        Console.WriteLine("Argument Error: index out of GPU entries");
                        return -1;
                    }

                    //set specified
                    var g = cm.GpuEntries[index.Value];
                    SetGpuFanSpeed(g);
                }
                else
                {
                    //set all
                    foreach (var g in cm.GpuEntries)
                    {
                        SetGpuFanSpeed(g);
                    }
                }
            }

            return 0;
        }

        private static ControlMemory ConnectToAfterburner(out int error)
        {
            Console.Write("Connecting to MSI Afterburner...");

            try
            {
                var cm = new ControlMemory();
                try
                {
                    var l = Console.CursorLeft;
                    Console.CursorLeft = 0;
                    Console.Write("".PadLeft(l));
                    Console.CursorLeft = 0;
                }
                catch (Exception e)
                {
                    Console.WriteLine();
                }

                error = 0;
                return cm;
            }
            catch (Exception e)
            {
                Console.WriteLine();
                Console.WriteLine(e.Message);
                error = -2;
                return null;
            }
        }

        private static void PrintUsage()
        {
            Console.WriteLine("GPU Fan Control Utility");
            Console.WriteLine("\tBased on MSI Afterburner 2.1 or later");
            Console.WriteLine();
            Console.WriteLine("Query GPU Fan Speed");
            Console.WriteLine("\t-q[index] blank for all");
            Console.WriteLine("\toutput: index\tmode\tmin\tval(%)\tmax");
            Console.WriteLine();
            Console.WriteLine("Set GPU Fan Speed");
            Console.WriteLine("\t-s<auto|percent>");
            Console.WriteLine("\t[-i<index>] blank for all");
            Console.WriteLine();
            Console.WriteLine("Exit Code:");
            Console.WriteLine("\tSuccess:\t0");
            Console.WriteLine("\tArgument Error:\t-1");
            Console.WriteLine("\tConnect Fail:\t-2");
            Console.WriteLine("\tUnknown Fail:\t-3");
        }
    }
}
